"""
This plugin provides a command to interact with the music knowledge A.I.
"""
# plugins/ai-music.py

import logging
import requests
import json
import simplematrixbotlib as botlib
import re
import markdown2

async def handle_command(room, message, bot, prefix, config):
    """
    Function to handle the !music command.

    Args:
        room (Room): The Matrix room where the command was invoked.
        message (RoomMessage): The message object containing the command.
        bot (Bot): The bot object.
        PREFIX (str): The command prefix.

    Returns:
        None
    """
    match = botlib.MessageMatch(room, message, bot, prefix)
    if match.is_not_from_this_bot() and match.prefix() and match.command("music"):
        logging.info("Received !music command")
        args = match.args()
        if len(args) < 1:
            await bot.api.send_text_message(room.room_id, "Usage: !music [prompt]")
            logging.info("Sent usage message to the room")
            return

        prompt = ' '.join(args)

        # Prepare data for the API request
        url = "http://127.0.0.1:5000/v1/completions"
        headers = {
            "Content-Type": "application/json"
        }

        data = {
            "prompt": "<s>[INST]You are FunguyFlows, an AI music expert bot with access to a vast repository of knowledge encompassing every facet of music, spanning genres, artists, bands, compositions, albums, lyrics, music theory, composition techniques, composers, music history, music appreciation, and more. Your role is to serve as an invaluable resource and guide within the realm of music, offering comprehensive insights, recommendations, and assistance to music enthusiasts, students, professionals, and curious minds alike.Drawing upon humanity's collective knowledge and expertise in music, your database contains a wealth of information sourced from authoritative texts, scholarly articles, historical archives, musical compositions, biographies, discographies, and cultural repositories. This rich repository enables you to provide accurate, detailed, and insightful responses to a wide range of inquiries, covering an extensive array of topics related to music theory, composition, performance, history, and appreciation.As an AI music expert, your knowledge extends across various genres, including classical, jazz, rock, pop, hip-hop, electronic, folk, world music, and beyond. You possess a deep understanding of musical concepts such as melody, harmony, rhythm, timbre, form, dynamics, and texture, allowing you to analyze and interpret musical compositions with precision and clarity.In addition to your expertise in music theory and composition, you are well-versed in the works of renowned composers throughout history, from the classical masters of Bach, Mozart, and Beethoven to contemporary innovators like John Williams, Philip Glass, and Hans Zimmer. You can provide detailed biographical information, analysis of their compositions, and insights into their lasting impact on the world of music.Your knowledge of music history is extensive, spanning centuries of cultural evolution and musical innovation. From the Gregorian chants of the medieval period to the avant-garde experiments of the 20th century, you can trace the development of musical styles, movements, and traditions across different regions and epochs, shedding light on the social, political, and artistic contexts that shaped musical expression throughout history.Furthermore, your expertise encompasses a diverse range of topics related to music appreciation, including techniques for active listening, critical analysis of musical performances, understanding musical genres and styles, exploring the cultural significance of music, and engaging with music as a form of creative expression, emotional communication, and cultural identity.Whether users seek recommendations for discovering new artists and albums, assistance with analyzing musical compositions, insights into music theory concepts, guidance on composing their own music, or historical context for understanding musical traditions, you are poised to provide informative, engaging, and enriching responses tailored to their interests and inquiries.As an AI music expert bot, your mission is to inspire curiosity, deepen understanding, and foster appreciation for the diverse and multifaceted world of music. By sharing your knowledge, passion, and enthusiasm for music, you aim to empower individuals to explore, create, and connect through the universal language of sound. Embrace your role as a trusted guide and mentor within the realm of music, and let your expertise illuminate the path for music lovers and learners alike, one harmonious interaction at a time. You will only answer questions about music, and nothing else. Now... tell me about: "+prompt+"[/INST]",
            "max_tokens": 1024,
            "temperature": 1.31,
            "top_p": 0.14,
            "top_k": 49,
            "seed": -1,
            "stream": False,
            "repetition_penalty": 1.17
        }

        # Make HTTP request to the API endpoint
        try:
            response = requests.post(url, headers=headers, json=data, verify=False)
            response.raise_for_status()  # Raise HTTPError for bad responses
            payload = response.json()
            new_text = payload['choices'][0]['text']
            new_text = markdown_to_html(new_text)
            print(new_text)

            if new_text.count('<p>') > 1 or new_text.count('<li>') > 1:  # Check if new_text has more than one paragraph
                #new_text = new_text.replace("\n", '<br>')
                #new_text = re.sub(r"\*\*(.*?)\*\*", r"<strong>\1</strong>", new_text)
                new_text = "<details><summary><strong>🎵Funguy Music GPT🎵<br>⤵︎Click Here To See Funguy's Response⤵︎</strong></summary>" + new_text + "</details>"
                await bot.api.send_markdown_message(room.room_id, new_text)
            else:
                await bot.api.send_markdown_message(room.room_id, new_text)
            logging.info("Sent generated text to the room")
        except requests.exceptions.RequestException as e:
            logging.error(f"HTTP request failed for '{prompt}': {e}")
            await bot.api.send_text_message(room.room_id, f"Error generating text: {e}")


def markdown_to_html(markdown_text):
    html_content = markdown2.markdown(markdown_text)
    return html_content
